@@ -1,25 +1,23 @@
-// Copyright (c) 2013, Kenton Varda <temporal@gmail.com>
-// All rights reserved.
+// Copyright (c) 2013-2014 Sandstorm Development Group, Inc. and contributors
+// Licensed under the MIT License:
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are met:
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// 1. Redistributions of source code must retain the above copyright notice, this
-//    list of conditions and the following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice,
-//    this list of conditions and the following disclaimer in the documentation
-//    and/or other materials provided with the distribution.
+// The above copyright notice and this permission notice shall be included in
+// all copies or substantial portions of the Software.
 //
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+// THE SOFTWARE.
 
 // Parser combinator framework!
 //
@@ -38,6 +36,10 @@
 #ifndef KJ_PARSE_COMMON_H_
 #define KJ_PARSE_COMMON_H_
 
+#if defined(__GNUC__) && !KJ_HEADER_WARNINGS
+#pragma GCC system_header
+#endif
+
 #include "../common.h"
 #include "../memory.h"
 #include "../array.h"
@@ -722,46 +724,6 @@ constexpr TransformWithLocation_<SubParser, TransformFunc> transformWithLocation
       kj::fwd<SubParser>(subParser), kj::fwd<TransformFunc>(functor));
 }
 
-// -------------------------------------------------------------------
-// acceptIf()
-// Output = Same as SubParser
-
-template <typename SubParser, typename Condition>
-class AcceptIf_ {
-public:
-  explicit constexpr AcceptIf_(SubParser&& subParser, Condition&& condition)
-      : subParser(kj::fwd<SubParser>(subParser)), condition(kj::fwd<Condition>(condition)) {}
-
-  template <typename Input>
-  Maybe<OutputType<SubParser, Input>> operator()(Input& input) const {
-    KJ_IF_MAYBE(subResult, subParser(input)) {
-      if (condition(*subResult)) {
-        return kj::mv(*subResult);
-      } else {
-        return nullptr;
-      }
-    } else {
-      return nullptr;
-    }
-  }
-
-private:
-  SubParser subParser;
-  Condition condition;
-};
-
-template <typename SubParser, typename Condition>
-constexpr AcceptIf_<SubParser, Condition> acceptIf(SubParser&& subParser, Condition&& condition) {
-  // Constructs a parser which executes some other parser and then invokes the functor
-  // `condition` on the result to check if it is valid.  Typically, `condition` is a lambda
-  // returning true or false.  Like with `transform()`, `condition` is invoked using `kj::apply`
-  // to unpack tuples.
-  //
-  // TODO(soon):  Remove in favor of transformOrReject()?
-  return AcceptIf_<SubParser, Condition>(
-      kj::fwd<SubParser>(subParser), kj::fwd<Condition>(condition));
-}
-
 // -------------------------------------------------------------------
 // notLookingAt()
 // Fails if the given parser succeeds at the current location.
